import middy, { MiddlewareObj } from '@middy/core';
import * as aws_lambda from 'aws-lambda';
import { APIGatewayProxyEvent, APIGatewayProxyResult, Context, ScheduledEvent } from 'aws-lambda';
import * as Joi from 'joi';
import { ObjectSchema } from 'joi';
import { BatchWriteCommandInput, BatchWriteCommandOutput, DeleteCommandInput, DeleteCommandOutput, GetCommandInput, GetCommandOutput, PutCommandInput, PutCommandOutput, QueryCommandInput, QueryCommandOutput, ScanCommandInput, ScanCommandOutput, UpdateCommandInput, UpdateCommandOutput } from '@aws-sdk/lib-dynamodb';
import * as type_fest from 'type-fest';

declare class HttpError extends Error {
    name: string;
    statusCode: number;
    constructor(messageOrError: string | Error, statusCode: number);
}

declare class ServiceError extends Error {
    name: string;
    code: number;
    statusCode: number;
    constructor(messageOrError: string | Error, code?: number, statusCode?: number);
}

declare class BadRequestError extends HttpError {
    name: string;
    constructor(message?: string);
}

declare class ForbiddenError extends HttpError {
    name: string;
    constructor(message?: string);
}

declare class NotFoundError extends HttpError {
    name: string;
    constructor(message?: string);
}

type HttpErrorHandlerOptions = {
    defaultMessage?: string;
    defaultStatusCode?: number;
    logger?(message: string): void;
};
declare const httpErrorHandler: (opts?: HttpErrorHandlerOptions) => MiddlewareObj<APIGatewayProxyEvent, APIGatewayProxyResult>;

/**
 * ValidatorOptions
 */
type ValidatorOptions = {
    eventSchema?: Joi.ObjectSchema;
    logger?(message: string): void;
};
/**
 * Validate Lambda events with a Joi schema.
 * @param options - Options for the validator.
 * @returns Middleware to perform validation against a Joi schema.
 */
declare const validator: (options: ValidatorOptions) => MiddlewareObj;

/**
 * The `LambdaConfig` type describes the configuration values supplied to every AWS Lambda
 * function by default. Extend `LambdaConfig` with your own service-specific configuration
 * type.
 *
 * Example:
 * ```
 * type ServiceConfig = LambdaConfig & {
 *   FOO: string;
 *   BAR: number;
 * }
 * ```
 */
type LambdaConfig = {
    AWS_EXECUTION_ENV: string;
    AWS_LAMBDA_FUNCTION_NAME: string;
    AWS_LAMBDA_FUNCTION_MEMORY_SIZE: string;
    AWS_LAMBDA_FUNCTION_VERSION: string;
    AWS_REGION: string;
};
/**
 * Validates the configuration with the supplied Joi ObjectSchema. Uses `process.env` as the
 * configuration source.
 * @param schema - A `Joi.ObjectSchema` of type `TConfig` to validate the configuration values.
 * @returns An Object of type `TConfig` containing the validated configuration if successful,
 * otherwise throws a `ServiceError`.
 * @throws Throws a `ServiceError` when validation is unsuccessful.
 */
declare function validateConfig<TConfig>(schema: Joi.ObjectSchema<TConfig>): TConfig;
/**
 * A Joi ObjectSchema to validate the base AWS Lambda function configuration values.
 */
declare const lambdaConfigSchema: Joi.ObjectSchema<any>;
/**
 * A `LambdaConfig` object containing the base AWS Lambda function configuration values.
 * This is useful when your function does not have additional configuration attributes.
 * @see {@link LambdaConfig}
 */
declare const lambdaConfigValues: LambdaConfig;

declare class DynamoService {
    private static instance;
    private client;
    private constructor();
    static getInstance(): DynamoService;
    batchWrite(input: BatchWriteCommandInput): Promise<BatchWriteCommandOutput>;
    delete(input: DeleteCommandInput): Promise<DeleteCommandOutput>;
    get(input: GetCommandInput): Promise<GetCommandOutput>;
    put(input: PutCommandInput): Promise<PutCommandOutput>;
    query(input: QueryCommandInput): Promise<QueryCommandOutput>;
    scan(input: ScanCommandInput): Promise<ScanCommandOutput>;
    update(input: UpdateCommandInput): Promise<UpdateCommandOutput>;
}

type APIGatewayHandlerFn = (event: APIGatewayProxyEvent, context: Context) => Promise<APIGatewayProxyResult>;
type ScheduledHandlerFn = (event: ScheduledEvent, context: Context) => Promise<void>;
type MiddyfyOptions<THandler> = {
    handler: THandler;
    logger?: (message: string) => void;
};
type APIGatewayMiddyfyOptions = MiddyfyOptions<APIGatewayHandlerFn> & {
    eventSchema?: ObjectSchema;
    defaultErrorMessage?: string;
    defaultErrorStatusCode?: number;
};
type ScheduledMiddyfyOptions = MiddyfyOptions<ScheduledHandlerFn> & {
    eventSchema?: ObjectSchema;
};
declare const middyfyAPIGateway: (options: APIGatewayMiddyfyOptions) => middy.MiddyfiedHandler<APIGatewayProxyEvent & Omit<APIGatewayProxyEvent, "body"> & {
    body: type_fest.JsonValue;
} & {
    multiValueQueryStringParameters: aws_lambda.APIGatewayProxyEventMultiValueQueryStringParameters;
    pathParameters: aws_lambda.APIGatewayProxyEventPathParameters;
    queryStringParameters: aws_lambda.APIGatewayProxyEventQueryStringParameters;
}, APIGatewayProxyResult, Error, Context>;
declare const middyfyScheduled: (options: ScheduledMiddyfyOptions) => middy.MiddyfiedHandler<ScheduledEvent<any>, void, Error, Context>;

export { type APIGatewayHandlerFn, type APIGatewayMiddyfyOptions, BadRequestError, DynamoService, ForbiddenError, HttpError, type HttpErrorHandlerOptions, type LambdaConfig, type MiddyfyOptions, NotFoundError, type ScheduledHandlerFn, type ScheduledMiddyfyOptions, ServiceError, type ValidatorOptions, httpErrorHandler, lambdaConfigSchema, lambdaConfigValues, middyfyAPIGateway, middyfyScheduled, validateConfig, validator };
